/*
 *	FGA - Fast Genetic Algorithm
 *	Copyright (C) 2007 Alessandro Presta (alessandro.presta@gmail.com)
 *	All rights reserved.
 *
 *	This library is free for commercial and non-commercial use as long as
 *	the following conditions are aheared to. The following conditions 
 *	apply to all code found in this distribution, be it the library itself
 *	or the included examples. All the documentation included with this 
 *	distribution is covered by the same copyright terms.
 *	
 *	Copyright remains Alessandro Presta's, and as such any Copyright 
 *	notices in the code are not to be removed.
 *	If this package is used in a product, Alessandro Presta should be 
 *	given attribution as the author of the parts of the library used.
 *	This can be in the form of a textual message at program startup or
 *	in documentation (online or textual) provided with the package.
 *
 *	Redistribution and use in source and binary forms, with or without
 *	modification, are permitted provided that the following conditions
 *	are met:
 *	1. Redistributions of source code must retain the copyright
 *	   notice, this list of conditions and the following disclaimer.
 *	2. Redistributions in binary form must reproduce the above copyright
 *	   notice, this list of conditions and the following disclaimer in the
 *	   documentation and/or other materials provided with the distribution.
 *	3. All advertising materials mentioning features or use of this software
 *	   must display the following acknowledgement:
 *	   "This product includes the Fast Genetic Algorithm library written by
 *	   Alessandro Presta (alessandro.presta@gmail.com)"
 *
 * 	THIS SOFTWARE IS PROVIDED BY ALESSANDRO PRESTA ``AS IS'' AND
 * 	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 *	FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * 	OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * 	HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * 	LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * 	OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * 	SUCH DAMAGE.
 * 
 * 	The licence and distribution terms for any publically available version or
 * 	derivative of this code cannot be changed.  i.e. this code cannot simply be
 * 	copied and put under another distribution licence
 *	(including the GNU Public Licence).
 */

#include <fga.hpp>
#include <cstdlib>
#include <ctime>
#include <iostream>

using namespace fga;
using namespace std;

#define N 100 // number of chromosomes in population
#define L 20 // number of genes in a chromosome

// fitness evaluation function
float my_fitness(bool *chromosome)
{
	float sum = 0;
	for (int i = 0; i < L; ++i)
		if (chromosome[i] == 1)
			++sum;
	return sum;	
}

// gene mutation function
bool my_mutate_gene(bool gene)
{
	if (gene == 1)
		return 0;
	else
		return 1;	
}

// binary tournament selection
bool *my_select(bool **c)
{
	int r1 = rand() % N;
	int r2 = rand() % N;
	if (my_fitness(c[r1]) > my_fitness(c[r2]))
		return c[r1];
	else
		return c[r2];	
}

// random gene creation function
bool my_random_gene()
{
  return rand() % 2;	
}

int main()
{
	srand(time(NULL));
	Population<bool> my_population(N, L, my_fitness, my_mutate_gene, my_random_gene, NULL, NULL, NULL, my_select);
	my_population.set_select_type(SELECT_TOURNAMENT);
	my_population.run();
	cout << "The population converged in " << my_population.get_generations() << " generations." << endl;
	cout << "Best chromosome: ";
	for (int i = 0; i < L; ++i)
		cout << my_population.get_best()[i];
	cout << endl;
	if (my_population.get_best_score() == L)
		cout << "Problem optimally solved." << endl;
	else
		cout << "Problem not optimally solved." << endl;
	return 0;
}
